<?php


class Payzen
{
    protected $boutique_id;
    protected $mode;
    protected $certif;
    protected $urlPlatForm;
    protected $urlNotify;
    protected $urlReturn;
    protected $urlCancel;
    protected $urlError;
    protected $btn;
    protected $autovalide;
    protected $nameboutique;
    protected $urlboutique;

    public function __construct()
    {
        $this->boutique_id = self::$config['boutique_id'];
        $this->mode = self::$config['mode'];
        $this->certif = self::$config['certif'];
        $this->urlPlatForm = self::$config['urlPlatForm'];
        $this->urlNotify = self::$config['urlNotify'];
        $this->urlReturn = self::$config['urlReturn'];
        $this->urlCancel = self::$config['urlCancel'];
        $this->urlError = self::$config['urlError'];
        $this->btn = self::$config['btn'];
        $this->autovalide = self::$config['autovalide'];
        if(!empty(self::$config['nameboutique'])) {
            $this->nameboutique = self::$config['nameboutique'];
        }
        if(!empty(self::$config['urlboutique'])) {
            $this->urlboutique = self::$config['urlboutique'];
        }
    }

    static protected $config = null;

    /**
     * Init la config de paypal
     *
     * @param array $config :
     *  - boutique_id : id de la boutique
     *  - certif : certid du certificat SSL
     *  - mode : TEST | PRODUCTION
     *  - urlPlatForm : The URL of the PayZen plat-form
     *  - urlNotify : The URL de notification
     *  - urlReturn : The URL de retour apres paiement
     *  - urlCancel : The URL de retour apres annulation
     *  - urlError : The URL de retour apres erreur de paiement
     *  - btn : image du bouton da validation
     *  - autovalide : valide automatiquement le formulaire (true | false)
     */
    static public function init($config = array()) {
        self::$config = $config;
    }

    /**
     * @param int $idReference
     * @param float $amount
     * @return mixed
     */
    public function request($idReference, $amount)
    {

        $params = array();


        $params['vads_site_id'] = $this->boutique_id;
        $params['vads_ctx_mode'] = $this->mode;
        $params['vads_trans_id'] = str_pad($idReference,6,'0',STR_PAD_LEFT);
        $params['vads_trans_date'] = date('YmdHis');
        $params['vads_amount'] = round($amount,2) * 100;
        $params['vads_currency'] = '978';
        $params['vads_action_mode'] = 'INTERACTIVE';
        $params['vads_page_action'] = 'PAYMENT';
        $params['vads_version'] = 'V2';
        $params['vads_payment_config'] = 'SINGLE';
        $params['vads_capture_delay'] = '0';
        $params['vads_validation_mode'] = '0';
        $params['vads_redirect_success_timeout'] = '0';
        $params['vads_redirect_success_message'] = '';
        $params['vads_redirect_error_timeout'] = '0';
        $params['vads_redirect_error_message'] = '';
        $params['vads_return_mode'] = 'POST';
        $params['vads_url_check'] = $this->urlNotify;
        $params['vads_url_return'] = $this->urlReturn;
        $params['vads_url_success"'] = $this->urlReturn;
        $params['vads_url_cancel'] = $this->urlCancel;
        $params['vads_url_refused'] = $this->urlError;
        $params['vads_url_error'] = $this->urlError;
        if(!empty($this->nameboutique)) {
            $params['vads_shop_name'] = $this->nameboutique;
        }
        if(!empty($this->urlboutique)) {
            $params['vads_shop_url'] = $this->urlboutique;
        }
        if(!empty($_SESSION['client_email'])) {
            $params['vads_cust_email'] = $_SESSION['client_email'];
        }
        if(!empty($_SESSION['client_id'])) {
            $params['vads_cust_id'] = $_SESSION['client_id'];
        }
        if(!empty($_SESSION['client_civilite'])) {
            $params['vads_cust_title'] = $_SESSION['client_civilite'];
        }
        if(!empty($_SESSION['client_prenom'])) {
            $params['vads_cust_first_name'] = $_SESSION['client_prenom'];
        }
        if(!empty($_SESSION['client_nom'])) {
            $params['vads_cust_last_name'] = $_SESSION['client_nom'];
        }
        if(!empty($_SESSION['commande']['livraison']['adresse_prenom'])) {
            $params['vads_ship_to_first_name'] = $_SESSION['commande']['livraison']['adresse_prenom'];
        }
        if(!empty($_SESSION['commande']['livraison']['adresse_nom'])) {
            $params['vads_ship_to_last_name'] = $_SESSION['commande']['livraison']['adresse_nom'];
        }
        if(!empty($_SESSION['commande']['livraison']['adresse_telephone'])) {
            $params['vads_ship_to_phone_num'] = $_SESSION['commande']['livraison']['adresse_telephone'];
        }
        if(!empty($_SESSION['commande']['livraison']['adresse_rue'])) {
            $params['vads_ship_to_street'] = $_SESSION['commande']['livraison']['adresse_rue'];
        }
        if(!empty($_SESSION['commande']['livraison']['adresse_rue2'])) {
            $params['vads_ship_to_street2'] = $_SESSION['commande']['livraison']['adresse_rue2'];
        }
        if(!empty($_SESSION['commande']['livraison']['adresse_cp'])) {
            $params['vads_ship_to_zip'] = $_SESSION['commande']['livraison']['adresse_cp'];
        }
        if(!empty($_SESSION['commande']['livraison']['adresse_ville'])) {
            $params['vads_ship_to_city'] = $_SESSION['commande']['livraison']['adresse_ville'];
        }
        if(!empty($_SESSION['commande']['livraison']['pays_iso_code_2'])) {
            $params['vads_ship_to_country'] = $_SESSION['commande']['livraison']['pays_iso_code_2'];
        }
        $params['signature'] = $this->sign($params);

        return $this->generateForm($params);

    }

    /**
     * Utility function, builds and returns the signature string of the data
     *  being transmitted to the PayZen plat-form
     *
     * @param $vads_form array, array of data being signed
     *
     * @return String, the signature
     */
    public function sign(Array $vads_form) {
        // Choice between TEST and PRODUCTION certificates
        $cert = $this->certif;
        ksort($vads_form);           // VADS values sorted by name, ascending order
        return sha1(                 // SHA1 encryption of ...
            implode('+', $vads_form)    // ... VADS array values joined with '+' ...
            . "+$cert"                  // ... concatenated with '+' and the certificate.
        );
    }

    /**
     * @param array
     * @return string
     */
    function generateForm($params) {
        $out ='<form action="'.$this->urlPlatForm.'" method="POST" enctype="UTF-8" accept-charset="multipart/form-data" id="formpayzen">';

        foreach($params as $key => $val) {
            $out .= '<input type="hidden" name="'.$key.'" value="'.$val.'" />';
        }
        $out .= '<input name="Valider" value="Valider" type="image" src="'.$this->btn.'" alt="Valider" id="valide_cb"/>'."\n";

        $out .= '</form>';

        if($this->autovalide) {
            $out.='<script type="text/javascript">jQuery(document).ready(function(){jQuery(\'#formpayzen\').submit()});</script>';
        }

        return $out;

    }

    /**
     * @param array $request
     * @return mixed
     */
    public function response($request)
    {
        if(empty($request) && !empty($_POST)) {
            $request = $_POST;
        }
        if(empty($request) && !empty($_GET)) {
            $request = $_GET;
        }
        if(empty($request)) {
            return false;
        }

        if(empty($request['vads_trans_id'])) {
            return false;
        }
        $order_id = $request['vads_trans_id'];

        $tpe = new Tpe;
        if(empty($request['vads_trans_status']) || $request['vads_trans_status'] != 'AUTHORISED') {
            $tpe->paiementKO($order_id, $request);
            return false;
        }

        if(empty($request['vads_auth_result']) || $request['vads_auth_result'] != '00') {
            $tpe->paiementKO($order_id, $request);
            return false;
        }

        $params = array();
        foreach($request as $key => $val) {
            if(substr($key,0,5) == 'vads_') {
                $params[$key] = $val;
            }
        }
        $signature = $this->sign($params);

        if(isset($request['signature']) && ($signature == $request['signature'])) {
            $tpe->paiementOK($order_id, $request);
            return false;
        } else {
            $tpe->paiementKO($order_id, $request);
            return false;
        }
    }


}